////////////////////////////////////////////////////////////////////////////
//
// nrMaterial.cpp
//
// A class for materials.
//
// Nate Robins, January 2002.
//
////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////
// Includes
////////////////////////////////////////////////////////////////////////////

#include "nrMaterial.h"

#include "nrParser.h"


////////////////////////////////////////////////////////////////////////////
// Public
////////////////////////////////////////////////////////////////////////////

nrMaterial::nrMaterial( const nrColor& ambient, const nrColor& diffuse, const nrColor& specular )
{
    m_Ambient = ambient;
    m_Diffuse = diffuse;
    m_Specular = specular;
}

////////////////////////////////////////////////////////////////////////////

nrMaterial::~nrMaterial( void )
{
}

////////////////////////////////////////////////////////////////////////////

nrMaterial* nrMaterial::Parse( nrParser& parser )
{
    nrColor ambient = nrColor( 0, 0, 0 );
    nrColor diffuse = nrColor( 1, 1, 1 );
    nrColor specular = nrColor( 0, 0, 0 );
    
    if ( parser.NextToken() == nrParser::TOKEN_STRING )
    {
        const char* name = parser.ReadString();
    }
    
    parser.ReadToken( nrParser::TOKEN_AGGREGATE_BEGIN );
    
    while ( parser.NextToken() == nrParser::TOKEN_KEY )
    {
        const char* key = parser.ReadKey();
        
        if ( parser.KeyMatches( key, "ambient" ) )
        {
            parser.ReadToken( nrParser::TOKEN_LIST_BEGIN );
            ambient.r = parser.ReadFloat();
            ambient.g = parser.ReadFloat();
            ambient.b = parser.ReadFloat();
            parser.ReadToken( nrParser::TOKEN_LIST_END );
        }
        else if ( parser.KeyMatches( key, "diffuse" ) )
        {
            parser.ReadToken( nrParser::TOKEN_LIST_BEGIN );
            diffuse.r = parser.ReadFloat();
            diffuse.g = parser.ReadFloat();
            diffuse.b = parser.ReadFloat();
            parser.ReadToken( nrParser::TOKEN_LIST_END );
        }
        else if ( parser.KeyMatches( key, "specular" ) )
        {
            parser.ReadToken( nrParser::TOKEN_LIST_BEGIN );
            specular.r = parser.ReadFloat();
            specular.g = parser.ReadFloat();
            specular.b = parser.ReadFloat();
            parser.ReadToken( nrParser::TOKEN_LIST_END );
        }
        else
        {
            parser.ParseError( "unknown key." );
        }
    }
    
    parser.ReadToken( nrParser::TOKEN_AGGREGATE_END );
    
    if ( parser.Error() )
    {
        return 0;
    }
    
    nrMaterial* material = new nrMaterial( ambient, diffuse, specular );
    
    return material;
}

////////////////////////////////////////////////////////////////////////////
