////////////////////////////////////////////////////////////////////////////
//
// nrModel3DS.h
//
// Nate Robins, November 2001.
//
// See http://astronomy.swin.edu.au/pbourke/3dformats/3ds/
//
// 3D Studio 3DS model file format reader.
//
////////////////////////////////////////////////////////////////////////////

#ifndef NRMODEL3DS_H
#define NRMODEL3DS_H


////////////////////////////////////////////////////////////////////////////
// Includes
////////////////////////////////////////////////////////////////////////////

#include <stdio.h>


////////////////////////////////////////////////////////////////////////////
// Classes
////////////////////////////////////////////////////////////////////////////

class nrVector2;
class nrVector3;
class nrmTriangle;
class nrmGroup;

////////////////////////////////////////////////////////////////////////////

class nrChunker;

class nrModel3DS
{
    friend class nrModel;
    
public:
    // Read a Wavefront 3DS model file.
    static nrModel* NewFromFile( const char* file_name );
    
private:
    nrModel3DS();
    ~nrModel3DS();
    
private:

    // Chunk processing functions.
    void Chunk_4D4D( nrChunker& chunker );
    void Chunk_3D3D( nrChunker& chunker );
    void Chunk_4000( nrChunker& chunker );
    void Chunk_4100( nrChunker& chunker );
    void Chunk_AFFF( nrChunker& chunker );
    
private:
    nrModel* m_Model;      // Model that we're populating.
    FILE* m_File;          // File from which model is being populated.

    char* m_Line;          // Current line in the file.
    int m_LineNum;         // Current line number in the file.
    
    int m_TotalBytes;      // Total number of bytes to be read.
    int m_NumBytes;        // Current number of bytes read.
};

////////////////////////////////////////////////////////////////////////////

#endif    // NRMODEL3DS_H
